( cd usr/lib64 ; rm -rf libSoundTouch.so )
( cd usr/lib64 ; ln -sf libSoundTouch.so.1.0.0 libSoundTouch.so )
( cd usr/lib64 ; rm -rf libSoundTouch.so.1 )
( cd usr/lib64 ; ln -sf libSoundTouch.so.1.0.0 libSoundTouch.so.1 )
( cd usr/lib64 ; rm -rf libSoundTouchDll.so )
( cd usr/lib64 ; ln -sf libSoundTouchDll.so.1.0.0 libSoundTouchDll.so )
( cd usr/lib64 ; rm -rf libSoundTouchDll.so.1 )
( cd usr/lib64 ; ln -sf libSoundTouchDll.so.1.0.0 libSoundTouchDll.so.1 )
